@@ -273,7 +273,7 @@ process_line__common (char     *line,
 	fr_command_message (comm, line);
 
 	if (comm->n_files != 0) {
-		double fraction = (double) comm->n_file++ / (comm->n_files+1);
+		double fraction = (double) ++comm->n_file / (comm->n_files + 1);
 		fr_command_progress (comm, fraction);
 	}
 }
